//! @todo Review this file.
/*!
LTC2484: 24-Bit Delta Sigma ADC with Easy Drive Input Current Cancellation

@verbatim

The LTC2484 combines a 24-bit no latency delta-sigma analog-to-digital 
converter with patented Easy Drive technology. The patented sampling scheme 
eliminates dynamic input current errors and the shortcomings of on-chip 
buffering through automatic cancellation of differential input current. This 
allows large external source impedances and input signals with rail-to-rail 
input range to be directly digitized while maintaining exceptional DC accuracy. 

The LTC2484 includes an on-chip oscillator. The LTC2484 can be configured to 
reject line frequencies. 50Hz, 60Hz or simultaneous 50Hz/60Hz line frequency 
rejection can be selected as well as a 2x speed-up mode. 

The LTC2484 allows a wide common mode input range (0V to VCC) independent of the 
reference voltage. The reference can be as low as 100mV or can be tied directly 
to VCC. The LTC2484 includes an on-chip trimmed oscillator, eliminating the need 
for external crystals or oscillators. Absolute accuracy and low drift are 
automatically maintained through continuous, transparent, offset and full-scale 
calibration. 

@endverbatim

REVISION HISTORY
$Revision: 2316 $
$Date: 2014-03-31 14:17:01 -0700 (Mon, 31 Mar 2014) $

Copyright (c) 2013, Linear Technology Corp.(LTC)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of Linear Technology Corp.

The Linear Technology Linduino is not affiliated with the official Arduino team.
However, the Linduino is only possible because of the Arduino team's commitment
to the open-source community.  Please, visit http://www.arduino.cc and
http://store.arduino.cc , and consider a purchase that will help fund their
ongoing work.
 */

//! @defgroup LTC2484 LTC2484: 24-Bit Delta Sigma ADC with Easy Drive Input Current Cancellation

/*! @file
    @ingroup LTC2484
    Library for LTC2484: 24-Bit Delta Sigma ADC with Easy Drive Input Current Cancellation
*/

#include <stdint.h>
#include <Arduino.h>
#include "Linduino.h"
#include "LT_SPI.h"
#include "LTC2484.h"
#include "LTC24XX_general.h"
#include <SPI.h>

// Checks for EOC with a specified timeout
int8_t LTC2484_EOC_timeout(uint8_t cs, uint16_t miso_timeout)
{

  return LTC24XX_EOC_timeout(cs, miso_timeout);
}
                         
void LTC2484_read(uint8_t cs, uint8_t adc_command, int32_t *adc_code)
// Reads the LTC2484
{
  LTC24XX_SPI_8bit_command_32bit_data(cs, adc_command, adc_code);
}

float LTC2484_code_to_voltage(int32_t adc_code, float LTC2484_lsb, int32_t LTC2484_offset_code)
// Calculates the LTC2484 input bipolar voltage
{
  return(LTC24XX_diff_code_to_calibrated_voltage(adc_code,  LTC2484_lsb, LTC2484_offset_code));
}

float LTC2484_temperature(int32_t adc_code, float LTC2484_t0, float LTC2484_r0)
// Calculate the LTC2484 temperature.
{
  adc_code -= 0x20000000;                                                   // Converts offset binary to binary
  return (((((float) adc_code) / LTC2484_r0) * (LTC2484_t0 + 273)) - 273);  // Calculate temperature from ADC code, t0, r0.
}

void LTC2484_cal_voltage(int32_t zero_code, int32_t fs_code, float zero_voltage, float fs_voltage, float *LTC2484_lsb, int32_t *LTC2484_offset_code)
// Calibrate the lsb
{
  zero_code -= 0x20000000;                                              // Converts zero code from offset binary to binary
  fs_code -= 0x20000000;                                                // Converts full scale from offset binary to binary
  
  float temp_offset;
  *LTC2484_lsb = (fs_voltage-zero_voltage)/((float)(fs_code - zero_code));                              // Calculate the LSB
  
  temp_offset = (zero_voltage/ *LTC2484_lsb) - zero_code;                                               // Calculate Unipolar offset
  temp_offset = (temp_offset > (floor(temp_offset) + 0.5)) ? ceil(temp_offset) : floor(temp_offset);    // Round
  *LTC2484_offset_code = (int32_t)temp_offset;                                                          // Cast as int32_t
}

void LTC2484_cal_temperature(int32_t adc_code, float temperature,  float *LTC2484_t0, float *LTC2484_r0)
// Calibrate temperature
{
  adc_code -= 0x20000000;              // Converts offset binary to binary
  *LTC2484_r0 = (float) adc_code;      // Convert the adc_code to a float value
  *LTC2484_t0 = temperature;           // Store the calibration temperature
}
